#!/usr/bin/env node
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

// Core
var commander = require('commander');
var app = require('express')();
var server = require('http').Server(app);
var cors = require('cors');
var bodyParser = require('body-parser');
// Helpers
var uuid = require('uuid');
var _ = require('lodash');

commander.version('0.0.1').option('-p, --port <port>', 'The port in which the server will run. Default to 3001.').parse(process.argv);

var port = 3001;

if (commander.port) {
  port = commander.port;
}

console.log('\uD83D\uDC80  Running RIP Todos on port ' + port);

// Todos State
var todos = [];
// Server port
server.listen(port);
// Applying middlewares
app.use(cors());
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: true }));
// API
// Get all todos
app.get('/todo/all', function (req, res) {
  res.json(todos);
});
// Get a single todo
app.get('/todo/:id', function (req, res) {
  var id = req.params.id;

  res.json(todos.find(function (todo) {
    return todo._id === id;
  }));
});
// Add a new todo
app.post('/todo', function (req, res) {
  try {
    var text = req.body.text;

    if (!text) {
      res.status(400).send('You must specify a text value to the todo');
    }
    todos = [].concat(_toConsumableArray(todos), [{
      _id: uuid.v4(),
      completed: false,
      text: text
    }]);
    res.status(200).end();
  } catch (e) {
    res.status(400).send('Bad Request');
  }
});
// Toggle a todo
app.patch('/todo', function (req, res) {
  var id = req.body.id;

  var oldTodos = todos;
  todos = todos.map(function (todo) {
    return todo._id !== id ? todo : _extends({}, todo, { completed: !todo.completed });
  });
  var statusCode = _.isEqual(todos, oldTodos) ? 400 : 200;
  res.status(statusCode).end();
});
// Delete a todo
app.delete('/todo/:id', function (req, res) {
  var id = req.param('id');
  var oldTodos = todos;
  todos = todos.filter(function (todo) {
    return todo._id !== id;
  });
  var statusCode = _.isEqual(todos, oldTodos) ? 400 : 200;
  res.status(statusCode).end();
});